use XML::Simple;

my $name_map = {};

sub get_permissions
{
	my $server_id = shift;
	my $prefix;
	if ( $server_id )
	{
		$prefix = "/data/$server_id";
	}
	# take in permissions xml
	my $xml = new XML::Simple;
	my $data = $xml->XMLin("$prefix/etc/asterisk/hud3.0/p3.xml");
	# this is required for uniformity between single and mutli-linked servers - Andre
	if(ref($data->{permissions}) eq "HASH") {
		$data->{permissions} = [$data->{permissions}];
	}
	return $data;
}

sub get_ast_hdrs
{
	my %info;

	# get asterisk headers
	while (<STDIN>)
	{
		$info{$1} = $2 if /^(.*): (.*)$/;
		dbg("$1 = $2");

		last if /^\r?\n/;
	}
	$info{agi_channel} =~ s/-.*$//;

	return %info;
}


sub ssend
{
	my $data = shift;
	print STDOUT "$data\n";
	my $line = <STDIN>;

	return $line;
}

# debug command
sub dbg
{
	my $log = shift;
	chomp($log);
	open F, ">>/tmp/agi.log";
	print F localtime() . ": $log\n";
	close(F);
}

# returns the CallerID (extension number) of the person paging
sub get_cid
{
	my $info = shift;

	my $cid = $info->{agi_callerid};
	$cid =~ s/^.*<(\d+)>$/$1/;

	if (!$cid)
	{
		dbg("no valid callerid");
	}

	return $cid;
}

# verify user has proper rights to perform function
sub check_rights
{
	my ($ref, $ext, $exts, $right_name, $must_be_group, $must_be_individual) = @_;

	set_name_map($ref);
	
	# find out who's allowed
	my %allowed;
	# we need to loop through multiple servers, possibly - Andre
	foreach my $srv (@{$ref->{permissions}}) {
		%allowed = (%allowed, map { $_ => 1 } get_exts_from_group($srv, $right_name));
	}
	
	# make sure we have permission
	foreach (keys %{$exts})
	{
		
		unless (
			($must_be_group      && $exts->{$_} == 1 && $allowed{$_}) ||
			($must_be_individual && $exts->{$_} == 0 && $allowed{$_}) ||
			(!$must_be_group && !$must_be_individual && $allowed{$_})
		)
		{
			# not allowed
			return 0;
		}
	}

	return 1;
}

sub set_name_map
{
	my $ref = shift;
	
	# map group ids to names, unfortunately there is ID uncomformity in the p3.xml
	# groups are labeled by their names, by a groups' group is labeled by its id - Andre
	foreach my $r (@{$ref->{permissions}}) {
		my $pos = $r->{group};
		my $one = exists $pos->{name};	
		foreach my $key (keys %{$pos}) {
			if($one) {
				$name_map{$pos->{id}} = $pos->{name};
				last;
			} else {
				$name_map{$pos->{$key}->{id}} = $key;
			}
		}
	}
}

# get extensions within allowed groups
sub get_exts_from_group
{
	my ($ref, $right_name) = @_;

	# which groups have this right?
	my (@groups, $loc, $type, $one);
	my $pos = $ref->{group};
	
	$one = exists $pos->{name};
	foreach my $key (keys %{$pos}) {
		# if there is a key 'name', it implies only 1 group`
		# this will break if someone has only 1 group called 'name'. Likelyhood is low,
		# since a user cannot delete group 'Basic Human Rights' - Andre
		if($one) {
			$loc = $pos->{permissions};
		} else {
			$loc = $pos->{$key}->{permissions};
		}
	
		my @permissions = ref($loc) eq "ARRAY" ? @$loc : ($loc);
		# search through each group's groups
		foreach (@permissions) {
			# search through the permissions associated
			foreach my $perm (split(",",$_->{ids})) {
				# is it there? # add the name of the group
				if($perm eq $right_name) {
					push(@groups, $name_map{$_->{group}});
					last;
				}
			}
		}

		last if $one;
	}	
	
	# get the extensions for each group
	return map { get_ref($ref, "group" . (!$one ? "->$_" : "") . "->members->ext") } @groups;
}

# sweet function to return a list of shiz from refs
sub get_ref
{
	my ($ref, @list) = @_;

	@list = map { split("->") } @list;
	
	my $var = $ref;
	foreach (@list)
	{
		if (ref($var) eq "HASH")
		{
			$var = $var->{$_};
		}
		else
		{
			$var = undef;
			last;
		}
	}

	my @list = ref($var) eq "ARRAY" ? @{$var} : ($var);
	foreach (@list)
	{
		s/R\((\d+),(\d+)\)/join(",", $1 .. $2)/eg;
	}
	return map { split(",") } @list;
}

# produce a list of extensions from extensions/extension groups
# returns a hash of extensions => 1 if in a group, => 0 if not, group extensions are removed
sub get_exts
{
	my ($ref, @exts) = @_;
	$ref ||= {};
	
	my $pos = $ref->{group} || {};
	my (%struct, $loc);
	
	# assume none of the exts are group extensions
	foreach my $ext (@exts) {
		$struct{$ext} = 0;
	}

	my $one = exists $pos->{name};	
	foreach my $key (keys %{$pos}) {
		# if there is a key 'name', it implies only 1 group`
		# this will break if someone has only 1 group called 'name'. Likelyhood is low,
		# since a user cannot delete group 'Basic Human Rights' - Andre
		if($one) {
			# is this a requested group extension?
			next unless(exists $struct{$pos->{ext}});
			delete $struct{$pos->{ext}};
			$loc = "members->ext";
		} else {
			# is this a requested group extension?
			next unless(exists $struct{$pos->{$key}->{ext}});
			delete $struct{$pos->{$key}->{ext}};
			$loc = "$key->members->ext";
		}
		
		foreach (get_ref($pos, $loc)) {
			# add all extensions assoicated with this group
			$struct{$_} = 1;
		}

		last if $one;
	}

	return \%struct;
}

1;
